/*
 * QEstEidCommon
 *
 * Copyright (C) 2010-2013 Jargo Kõster <jargo@innovaatik.ee>
 * Copyright (C) 2010-2013 Raul Metsma <raul@innovaatik.ee>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 */

#include "Common.h"

#include <Cocoa/Cocoa.h>

@implementation NSApplication (ApplicationObjC)

- (void)appReopen:(NSAppleEventDescriptor *)event withReplyEvent:(NSAppleEventDescriptor *)replyEvent
{
	Q_UNUSED(event)
	Q_UNUSED(replyEvent)
	QApplication::postEvent( qApp, new REOpenEvent );
}

- (void)openClient:(NSPasteboard *)pboard userData:(NSString *)data error:(NSString **)error
{
	Q_UNUSED(data)
	Q_UNUSED(error)
	QStringList result;
	for( NSString *filename in [pboard propertyListForType:NSFilenamesPboardType] )
	{
		QString file( [filename length], 0 );
		[filename getCharacters:reinterpret_cast<unichar*>(file.data()) range:NSMakeRange(0, [filename length])];
		result << file;
	}
	QMetaObject::invokeMethod( qApp, "showClient", Q_ARG(QStringList,result) );
}

- (void)openCrypto:(NSPasteboard *)pboard userData:(NSString *)data error:(NSString **)error
{
	Q_UNUSED(data)
	Q_UNUSED(error)
	QStringList result;
	for( NSString *filename in [pboard propertyListForType:NSFilenamesPboardType] )
	{
		QString file( [filename length], 0 );
		[filename getCharacters:reinterpret_cast<unichar*>(file.data()) range:NSMakeRange(0, [filename length])];
		result << file;
	}
	QMetaObject::invokeMethod( qApp, "showCrypto", Q_ARG(QStringList,result) );
}
@end

void Common::addRecent( const QString &file )
{
	NSString *name = [NSString stringWithCharacters:reinterpret_cast<const UniChar *>(file.unicode()) length:file.length()];
	[[NSDocumentController sharedDocumentController] noteNewRecentDocumentURL:[NSURL fileURLWithPath:name]];
}

void Common::initMacEvents()
{
	if( macEvents )
		return;
	[[NSAppleEventManager sharedAppleEventManager] setEventHandler:NSApp
		andSelector:@selector(appReopen:withReplyEvent:)
		forEventClass:kCoreEventClass
		andEventID:kAEReopenApplication];
	// reload /System/Library/CoreServices/pbs
	// list /System/Library/CoreServices/pbs -dump_pboard
	[NSApp setServicesProvider:NSApp];
	macEvents = true;
}

void Common::deinitMacEvents()
{
	[[NSAppleEventManager sharedAppleEventManager]
		removeEventHandlerForEventClass:kCoreEventClass
		andEventID:kAEReopenApplication];
}
